use crate::read;

#[derive(Debug, PartialEq, Eq, Clone, Hash)]
enum Command {
    AddX(i32),
    Noop,
}
impl Command {
    fn from_str(s: &str) -> Self {
        if s == "noop" {
            Self::Noop
        } else {
            let sp = s.split_once(' ').unwrap();
            Self::AddX(sp.1.parse().unwrap())
        }
    }
}

pub fn run() {
    const LINE_WIDTH: usize = 40;
    const SPRITE_WIDTH: usize = 3;
    const SPRITE_WIDTH_HALF: usize = SPRITE_WIDTH / 2;

    let input = read("10");
    let mut lines = input.lines();

    let mut cycle = 1;
    let mut is_busy = 0;
    let mut v: i32 = 1;
    let mut v_new: i32 = 1;
    let mut sig_str = vec![];
    let mut pixels = [false; LINE_WIDTH];
    let mut rows = vec![];
    'outer: loop {
        if cycle == 20 || (cycle > 20 && (cycle - 20) % LINE_WIDTH == 0) {
            sig_str.push(cycle as i32 * v);
        }
        if is_busy == 0 {
            match lines.next() {
                Some(s) => {
                    let cmd = Command::from_str(s);
                    match cmd {
                        Command::Noop => {}
                        Command::AddX(x) => {
                            v_new += x;
                            is_busy = 1;
                        }
                    }
                }
                _ => {
                    break 'outer;
                }
            };
        } else {
            is_busy -= 1;
        }

        let pixel = (cycle - 1) % LINE_WIDTH;
        let is_lit = pixel.abs_diff(v as usize) <= SPRITE_WIDTH_HALF;
        pixels[pixel] = is_lit;
        if pixel == LINE_WIDTH - 1 {
            let row: String = pixels.map(|l| if l { '#' } else { ' ' }).iter().collect();
            rows.push(row);
        }

        if is_busy == 0 {
            v = v_new;
        }
        cycle += 1;
    }

    #[cfg(feature = "part1")]
    {
        println!("Day 10, Part 01: {}", sig_str.iter().sum::<i32>());
    }

    #[cfg(feature = "part2")]
    {
        println!("Day 10, Part 02: \n{}", rows.join("\n"));
    }
}
